/* GCompris - colors.js
 *
 * Copyright (C) 2015 Bruno Coudoin
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, see <https://www.gnu.org/licenses/>.
 */

function get() {
    return [
            [ // Level 1
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/yellow_duck.svg",
                 "text": qsTr("Find the yellow duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/yellow.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/black_duck.svg",
                 "text": qsTr("Find the black duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/black.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/green_duck.svg",
                 "text": qsTr("Find the green duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/green.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/red_duck.svg",
                 "text": qsTr("Find the red duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/red.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/white_duck.svg",
                 "text": qsTr("Find the white duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/white.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/blue_duck.svg",
                 "text": qsTr("Find the blue duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/blue.$CA")
             }
            ],
            [ // Level 2
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/yellow_duck.svg",
                 "text": qsTr("Find the yellow duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/yellow.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/black_duck.svg",
                 "text": qsTr("Find the black duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/black.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/brown_duck.svg",
                 "text": qsTr("Find the brown duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/brown.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/green_duck.svg",
                 "text": qsTr("Find the green duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/green.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/grey_duck.svg",
                 "text": qsTr("Find the grey duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/grey.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/orange_duck.svg",
                 "text": qsTr("Find the orange duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/orange.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/purple_duck.svg",
                 "text": qsTr("Find the purple duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/purple.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/white_duck.svg",
                 "text": qsTr("Find the white duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/white.$CA")
             }
            ],
            [ // Level 3
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/yellow_duck.svg",
                 "text": qsTr("Find the yellow duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/yellow.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/brown_duck.svg",
                 "text": qsTr("Find the brown duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/brown.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/green_duck.svg",
                 "text": qsTr("Find the green duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/green.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/grey_duck.svg",
                 "text": qsTr("Find the grey duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/grey.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/orange_duck.svg",
                 "text": qsTr("Find the orange duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/orange.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/purple_duck.svg",
                 "text": qsTr("Find the purple duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/purple.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/pink_duck.svg",
                 "text": qsTr("Find the pink duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/pink.$CA")
             },
             {
                 "image": "qrc:/gcompris/src/activities/colors/resource/blue_duck.svg",
                 "text": qsTr("Find the blue duck"),
                 "audio": ApplicationInfo.getAudioFilePath("voices-$CA/$LOCALE/colors/blue.$CA")
             }
            ]
        ];
}
