/* GCompris
 *
 * Copyright (C) 2014 Bruno Coudoin
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, see <https://www.gnu.org/licenses/>.
 */

var advanced_colors =
        [
            [ // Level 1
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/coral_butterfly.svg",
                 //: Look at https://gcompris.net/wiki/Advanced_color_translation to see all the advanced colors
                 //: Color #FF7F50
                 "text": qsTr("Find the coral butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/claret_butterfly.svg",
                 //: Color #7F1734
                 "text": qsTr("Find the claret butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/navy_butterfly.svg",
                 //: Color #000080
                 "text": qsTr("Find the navy butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/corn_butterfly.svg",
                 //: Color #FBEC5D
                 "text": qsTr("Find the corn butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/cobalt_butterfly.svg",
                 //: Color #0047AB
                 "text": qsTr("Find the cobalt butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/cyan_butterfly.svg",
                 //: Color #00FFFF
                 "text": qsTr("Find the cyan butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/chestnut_butterfly.svg",
                 //: Color #954535
                 "text": qsTr("Find the chestnut butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/almond_butterfly.svg",
                 //: Color #AB784E
                 "text": qsTr("Find the almond butterfly")
             }
            ],
            [ // Level 2
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/claret_butterfly.svg",
                 //: Color #7F1734
                 "text": qsTr("Find the claret butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/sapphire_butterfly.svg",
                 //: Color #0F52BA
                 "text": qsTr("Find the sapphire butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/corn_butterfly.svg",
                 //: Color #FBEC5D
                 "text": qsTr("Find the corn butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/cobalt_butterfly.svg",
                 //: Color #0047AB
                 "text": qsTr("Find the cobalt butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/cyan_butterfly.svg",
                 //: Color #00FFFF
                 "text": qsTr("Find the cyan butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/chestnut_butterfly.svg",
                 //: Color #954535
                 "text": qsTr("Find the chestnut butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/almond_butterfly.svg",
                 //: Color #AB784E
                 "text": qsTr("Find the almond butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/ruby_butterfly.svg",
                 //: Color #E0115F
                 "text": qsTr("Find the ruby butterfly")
             }
            ],
            [ // Level 3
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/sienna_butterfly.svg",
                 //: Color #882D17
                 "text": qsTr("Find the sienna butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/sage_butterfly.svg",
                 //: Color #BCB88A
                 "text": qsTr("Find the sage butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/salmon_butterfly.svg",
                 //: Color #FF8C69
                 "text": qsTr("Find the salmon butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/sepia_butterfly.svg",
                 //: Color #704214
                 "text": qsTr("Find the sepia butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/sulphur_butterfly.svg",
                 //: Color #E4BB25
                 "text": qsTr("Find the sulfur butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/tea_butterfly.svg",
                 //: Color #DB6D7B
                 "text": qsTr("Find the tea butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/lime_butterfly.svg",
                 //: Color #BFFF00
                 "text": qsTr("Find the lime butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/turquoise_butterfly.svg",
                 //: Color #40E0D0
                 "text": qsTr("Find the turquoise butterfly")
             }
            ],
            [ // Level 4
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/absinthe_butterfly.svg",
                 //: Color #73B881
                 "text": qsTr("Find the absinthe butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/mahogany_butterfly.svg",
                 //: Color #C04000
                 "text": qsTr("Find the mahogany butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/aquamarine_butterfly.svg",
                 //: Color #7FFFD4
                 "text": qsTr("Find the aquamarine butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/alabaster_butterfly.svg",
                 //: Color #F2F0E6
                 "text": qsTr("Find the alabaster butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/amber_butterfly.svg",
                 //: Color #FFBF00
                 "text": qsTr("Find the amber butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/amethyst_butterfly.svg",
                 //: Color #9966CC
                 "text": qsTr("Find the amethyst butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/anise_butterfly.svg",
                 //: Color #F5EACC
                 "text": qsTr("Find the anise butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/vermilion_butterfly.svg",
                 //: Color #E34234
                 "text": qsTr("Find the vermilion butterfly")
             }
            ],
            [ // Level 5
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/ceruse_butterfly.svg",
                 //: Color #EEEBEB
                 "text": qsTr("Find the ceruse butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/fawn_butterfly.svg",
                 //: Color #E5AA70
                 "text": qsTr("Find the fawn butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/chartreuse_butterfly.svg",
                 //: Color #7FFF00
                 "text": qsTr("Find the chartreuse butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/emerald_butterfly.svg",
                 //: Color #50C878
                 "text": qsTr("Find the emerald butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/aubergine_butterfly.svg",
                 //: Color #614051
                 "text": qsTr("Find the aubergine butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/fuchsia_butterfly.svg",
                 //: Color #CC397B
                 "text": qsTr("Find the fuchsia butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/ruby_butterfly.svg",
                 //: Color #E0115F
                 "text": qsTr("Find the ruby butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/glaucous_butterfly.svg",
                 //: Color #6082B6
                 "text": qsTr("Find the glaucous butterfly")
             }
            ],
            [ // Level 6
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/aubergine_butterfly.svg",
                 //: Color #614051
                 "text": qsTr("Find the aubergine butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/auburn_butterfly.svg",
                 //: Color #A52A2A
                 "text": qsTr("Find the auburn butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/azure_butterfly.svg",
                 //: Color #458AC6
                 "text": qsTr("Find the azure butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/greyish-brown_butterfly.svg",
                 //: Color #9F7F58
                 "text": qsTr("Find the grayish brown butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/bistre_butterfly.svg",
                 //: Color #3D2B1F
                 "text": qsTr("Find the bistre butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/crimson_butterfly.svg",
                 //: Color #DC143C
                 "text": qsTr("Find the crimson butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/celadon_butterfly.svg",
                 //: Color #ACE1AF
                 "text": qsTr("Find the celadon butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/cerulean_butterfly.svg",
                 //: Color #007BA7
                 "text": qsTr("Find the cerulean butterfly")
             }
            ],
            [ // Level 7
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/dove_butterfly.svg",
                 //: Color #8B8E8D
                 "text": qsTr("Find the dove butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/garnet_butterfly.svg",
                 //: Color #943543
                 "text": qsTr("Find the garnet butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/indigo_butterfly.svg",
                 //: Color #4B0082
                 "text": qsTr("Find the indigo butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/ivory_butterfly.svg",
                 //: Color #FFFFF0
                 "text": qsTr("Find the ivory butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/jade_butterfly.svg",
                 //: Color #00A86B
                 "text": qsTr("Find the jade butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/lavender_butterfly.svg",
                 //: Color #B57EDC
                 "text": qsTr("Find the lavender butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/lichen_butterfly.svg",
                 //: Color #9BC4AF
                 "text": qsTr("Find the lichen butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/wine_butterfly.svg",
                 //: Color #722F37
                 "text": qsTr("Find the wine butterfly")
             }
            ],
            [ // Level 8
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/lilac_butterfly.svg",
                 //: Color #9955BB
                 "text": qsTr("Find the lilac butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/magenta_butterfly.svg",
                 //: Color #FF00FF
                 "text": qsTr("Find the magenta butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/malachite_butterfly.svg",
                 //: Color #0BDA51
                 "text": qsTr("Find the malachite butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/navy_butterfly.svg",
                 //: Color #000080
                 "text": qsTr("Find the navy butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/larch_butterfly.svg",
                 //: Color #D9C3AD
                 "text": qsTr("Find the larch butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/mimosa_butterfly.svg",
                 //: Color #EFC050
                 "text": qsTr("Find the mimosa butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/ochre_butterfly.svg",
                 //: Color #CC7722
                 "text": qsTr("Find the ochre butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/olive_butterfly.svg",
                 //: Color #808000
                 "text": qsTr("Find the olive butterfly")
             }
            ],
            [ // Level 9
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/opaline_butterfly.svg",
                 //: Color #F7F9F4
                 "text": qsTr("Find the opaline butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/vermilion_butterfly.svg",
                 //: Color #E34234
                 "text": qsTr("Find the vermilion butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/ultramarine_butterfly.svg",
                 //: Color #120A8F
                 "text": qsTr("Find the ultramarine butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/mauve_butterfly.svg",
                 //: Color #E0B0FF
                 "text": qsTr("Find the mauve butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/greyish_blue_butterfly.svg",
                 //: Color #91A3B0
                 "text": qsTr("Find the grayish blue butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/pistachio_butterfly.svg",
                 //: Color #93C572
                 "text": qsTr("Find the pistachio butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/platinum_butterfly.svg",
                 //: Color #AEADA5
                 "text": qsTr("Find the platinum butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/purple_butterfly.svg",
                 //: Color #32004A
                 "text": qsTr("Find the purple butterfly")
             }
            ],
            [ // Level 10
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/plum_butterfly.svg",
                 //: Color #5C3960
                 "text": qsTr("Find the plum butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/prussian_blue_butterfly.svg",
                 //: Color #003153
                 "text": qsTr("Find the prussian blue butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/rust_butterfly.svg",
                 //: Color #B7410E
                 "text": qsTr("Find the rust butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/saffron_butterfly.svg",
                 //: Color #F4C430
                 "text": qsTr("Find the saffron butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/vanilla_butterfly.svg",
                 //: Color #F3E5AB
                 "text": qsTr("Find the vanilla butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/veronese_butterfly.svg",
                 //: Color #40826D
                 "text": qsTr("Find the veronese butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/verdigris_butterfly.svg",
                 //: Color #529371
                 "text": qsTr("Find the verdigris butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/dark_purple_butterfly.svg",
                 //: Color #32004A
                 "text": qsTr("Find the dark purple butterfly")
             }
            ],
            [ // Level 11 Now all the colors have been seen, propose here
             // a new random set of them
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/olive_butterfly.svg",
                 //: Color #808000
                 "text": qsTr("Find the olive butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/auburn_butterfly.svg",
                 //: Color #A52A2A
                 "text": qsTr("Find the auburn butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/dark_purple_butterfly.svg",
                 //: Color #32004A
                 "text": qsTr("Find the dark purple butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/mauve_butterfly.svg",
                 //: Color #E0B0FF
                 "text": qsTr("Find the mauve butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/azure_butterfly.svg",
                 //: Color #458AC6
                 "text": qsTr("Find the azure butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/plum_butterfly.svg",
                 //: Color #5C3960
                 "text": qsTr("Find the plum butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/sepia_butterfly.svg",
                 //: Color #704214
                 "text": qsTr("Find the sepia butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/aubergine_butterfly.svg",
                 //: Color #614051
                 "text": qsTr("Find the aubergine butterfly")
             }
            ],
            [ // Level 12
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/aubergine_butterfly.svg",
                 //: Color #614051
                 "text": qsTr("Find the aubergine butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/cyan_butterfly.svg",
                 //: Color #00FFFF
                 "text": qsTr("Find the cyan butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/fawn_butterfly.svg",
                 //: Color #E5AA70
                 "text": qsTr("Find the fawn butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/mimosa_butterfly.svg",
                 //: Color #EFC050
                 "text": qsTr("Find the mimosa butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/ochre_butterfly.svg",
                 //: Color #CC7722
                 "text": qsTr("Find the ochre butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/prussian_blue_butterfly.svg",
                 //: Color #003153
                 "text": qsTr("Find the prussian blue butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/rust_butterfly.svg",
                 //: Color #B7410E
                 "text": qsTr("Find the rust butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/salmon_butterfly.svg",
                 //: Color #FF8C69
                 "text": qsTr("Find the salmon butterfly")
             }
            ],
            [ // Level 13
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/alabaster_butterfly.svg",
                 //: Color #F2F0E6
                 "text": qsTr("Find the alabaster butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/chestnut_butterfly.svg",
                 //: Color #954535
                 "text": qsTr("Find the chestnut butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/coral_butterfly.svg",
                 //: Color #FF7F50
                 "text": qsTr("Find the coral butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/ivory_butterfly.svg",
                 //: Color #FFFFF0
                 "text": qsTr("Find the ivory butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/mauve_butterfly.svg",
                 //: Color #E0B0FF
                 "text": qsTr("Find the mauve butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/plum_butterfly.svg",
                 //: Color #5C3960
                 "text": qsTr("Find the plum butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/rust_butterfly.svg",
                 //: Color #B7410E
                 "text": qsTr("Find the rust butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/vermilion_butterfly.svg",
                 //: Color #E34234
                 "text": qsTr("Find the vermilion butterfly")
             }
            ],
            [ // Level 14
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/navy_butterfly.svg",
                 //: Color #000080
                 "text": qsTr("Find the navy butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/cerulean_butterfly.svg",
                 //: Color #007BA7
                 "text": qsTr("Find the cerulean butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/malachite_butterfly.svg",
                 //: Color #0BDA51
                 "text": qsTr("Find the malachite butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/opaline_butterfly.svg",
                 //: Color #F7F9F4
                 "text": qsTr("Find the opaline butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/wine_butterfly.svg",
                 //: Color #722F37
                 "text": qsTr("Find the wine butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/pistachio_butterfly.svg",
                 //: Color #93C572
                 "text": qsTr("Find the pistachio butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/chestnut_butterfly.svg",
                 //: Color #954535
                 "text": qsTr("Find the chestnut butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/claret_butterfly.svg",
                 //: Color #7F1734
                 "text": qsTr("Find the claret butterfly")
             }
            ],
            [ // Level 15
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/chestnut_butterfly.svg",
                 //: Color #954535
                 "text": qsTr("Find the chestnut butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/dark_purple_butterfly.svg",
                 //: Color #32004A
                 "text": qsTr("Find the dark purple butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/aubergine_butterfly.svg",
                 //: Color #614051
                 "text": qsTr("Find the aubergine butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/opaline_butterfly.svg",
                 //: Color #F7F9F4
                 "text": qsTr("Find the opaline butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/wine_butterfly.svg",
                 //: Color #722F37
                 "text": qsTr("Find the wine butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/fawn_butterfly.svg",
                 //: Color #E5AA70
                 "text": qsTr("Find the fawn butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/magenta_butterfly.svg",
                 //: Color #FF00FF
                 "text": qsTr("Find the magenta butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/anise_butterfly.svg",
                 //: Color #F5EACC
                 "text": qsTr("Find the anise butterfly")
             }
            ],
            [ // Level 16
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/claret_butterfly.svg",
                 //: Color #7F1734
                 "text": qsTr("Find the claret butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/fawn_butterfly.svg",
                 //: Color #E5AA70
                 "text": qsTr("Find the fawn butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/platinum_butterfly.svg",
                 //: Color #AEADA5
                 "text": qsTr("Find the platinum butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/indigo_butterfly.svg",
                 //: Color #4B0082
                 "text": qsTr("Find the indigo butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/dove_butterfly.svg",
                 //: Color #8B8E8D
                 "text": qsTr("Find the dove butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/crimson_butterfly.svg",
                 //: Color #DC143C
                 "text": qsTr("Find the crimson butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/veronese_butterfly.svg",
                 //: Color #40826D
                 "text": qsTr("Find the veronese butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/greyish-brown_butterfly.svg",
                 //: Color #9F7F58
                 "text": qsTr("Find the grayish brown butterfly")
             }
            ],
            [ // Level 17
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/chestnut_butterfly.svg",
                 //: Color #954535
                 "text": qsTr("Find the chestnut butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/greyish_blue_butterfly.svg",
                 //: Color #91A3B0
                 "text": qsTr("Find the grayish blue butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/aubergine_butterfly.svg",
                 //: Color #614051
                 "text": qsTr("Find the aubergine butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/salmon_butterfly.svg",
                 //: Color #FF8C69
                 "text": qsTr("Find the salmon butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/amber_butterfly.svg",
                 //: Color #FFBF00
                 "text": qsTr("Find the amber butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/chartreuse_butterfly.svg",
                 //: Color #7FFF00
                 "text": qsTr("Find the chartreuse butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/claret_butterfly.svg",
                 //: Color #7F1734
                 "text": qsTr("Find the claret butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/purple_butterfly.svg",
                 //: Color #32004A
                 "text": qsTr("Find the purple butterfly")
             }
            ],
            [ // Level 18
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/plum_butterfly.svg",
                 //: Color #5C3960
                 "text": qsTr("Find the plum butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/claret_butterfly.svg",
                 //: Color #7F1734
                 "text": qsTr("Find the claret butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/ultramarine_butterfly.svg",
                 //: Color #120A8F
                 "text": qsTr("Find the ultramarine butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/greyish_blue_butterfly.svg",
                 //: Color #91A3B0
                 "text": qsTr("Find the grayish blue butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/dark_purple_butterfly.svg",
                 //: Color #32004A
                 "text": qsTr("Find the dark purple butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/lilac_butterfly.svg",
                 //: Color #9955BB
                 "text": qsTr("Find the lilac butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/aquamarine_butterfly.svg",
                 //: Color #7FFFD4
                 "text": qsTr("Find the aquamarine butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/magenta_butterfly.svg",
                 //: Color #FF00FF
                 "text": qsTr("Find the magenta butterfly")
             }
            ],
            [ // Level 19
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/anise_butterfly.svg",
                 //: Color #F5EACC
                 "text": qsTr("Find the anise butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/cerulean_butterfly.svg",
                 //: Color #007BA7
                 "text": qsTr("Find the cerulean butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/ivory_butterfly.svg",
                 //: Color #FFFFF0
                 "text": qsTr("Find the ivory butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/sepia_butterfly.svg",
                 //: Color #704214
                 "text": qsTr("Find the sepia butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/verdigris_butterfly.svg",
                 //: Color #529371
                 "text": qsTr("Find the verdigris butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/fuchsia_butterfly.svg",
                 //: Color #CC397B
                 "text": qsTr("Find the fuchsia butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/plum_butterfly.svg",
                 //: Color #5C3960
                 "text": qsTr("Find the plum butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/purple_butterfly.svg",
                 //: Color #32004A
                 "text": qsTr("Find the purple butterfly")
             }
            ],
            [ // Level 20
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/tea_butterfly.svg",
                 //: Color #DB6D7B
                 "text": qsTr("Find the tea butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/dark_purple_butterfly.svg",
                 //: Color #32004A
                 "text": qsTr("Find the dark purple butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/sienna_butterfly.svg",
                 //: Color #882D17
                 "text": qsTr("Find the sienna butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/aubergine_butterfly.svg",
                 //: Color #614051
                 "text": qsTr("Find the aubergine butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/rust_butterfly.svg",
                 //: Color #B7410E
                 "text": qsTr("Find the rust butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/pistachio_butterfly.svg",
                 //: Color #93C572
                 "text": qsTr("Find the pistachio butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/sapphire_butterfly.svg",
                 //: Color #0F52BA
                 "text": qsTr("Find the sapphire butterfly")
             },
             {
                 "image": "qrc:/gcompris/src/activities/advanced_colors/resource/claret_butterfly.svg",
                 //: Color #7F1734
                 "text": qsTr("Find the claret butterfly")
             }
            ]
        ]


function get() {
    return advanced_colors
}
